//
// Copyright (C) 2010 Wayne Meissner
// Copyright (c) 2008-2009, Petr Kobalicek <kobalicek.petr@gmail.com>
//
// Permission is hereby granted, free of charge, to any person
// obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without
// restriction, including without limitation the rights to use,
// copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following
// conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.

package com.kenai.jnr.x86asm;

@Deprecated
public class RID {

    private RID() {
    }
    /** ID for AX/EAX/RAX registers. */
    public static final int RID_EAX = 0;
    /** ID for CX/ECX/RCX registers. */
    public static final int RID_ECX = 1;
    /** ID for DX/EDX/RDX registers. */
    public static final int RID_EDX = 2;
    /** ID for BX/EBX/RBX registers. */
    public static final int RID_EBX = 3;
    /** ID for SP/ESP/RSP registers. */
    public static final int RID_ESP = 4;
    /** ID for BP/EBP/RBP registers. */
    public static final int RID_EBP = 5;
    /** ID for SI/ESI/RSI registers. */
    public static final int RID_ESI = 6;
    /** ID for DI/EDI/RDI registers. */
    public static final int RID_EDI = 7;
}
